@@ -1,7 +1,4 @@
-use std::{
-    convert::TryInto,
-    ops::{Deref, DerefMut},
-};
+use std::convert::TryInto;
 
 use git_actor as actor;
 use git_hash::ObjectId;
@@ -14,24 +11,24 @@ use git_ref::{
 use crate::{
     bstr::BString,
     easy,
-    easy::{ext::RepositoryAccessExt, reference, Reference},
+    easy::{reference, Reference},
     ext::ReferenceExt,
 };
 
 const DEFAULT_LOCK_MODE: git_lock::acquire::Fail = git_lock::acquire::Fail::Immediately;
 
 /// Obtain and alter references comfortably
-pub trait ReferenceAccessExt: easy::Access + Sized {
+impl easy::Handle {
     /// Create a lightweight tag with given `name` (and without `refs/tags/` prefix) pointing to the given `target`, and return it as reference.
     ///
     /// It will be created with `constraint` which is most commonly to [only create it][PreviousValue::MustNotExist]
     /// or to [force overwriting a possibly existing tag](PreviousValue::Any).
-    fn tag_reference(
+    pub fn tag_reference(
         &self,
         name: impl AsRef<str>,
         target: impl Into<ObjectId>,
         constraint: PreviousValue,
-    ) -> Result<Reference<'_, Self>, reference::edit::Error> {
+    ) -> Result<Reference<'_>, reference::edit::Error> {
         let id = target.into();
         let mut edits = self.edit_reference(
             RefEdit {
@@ -54,33 +51,35 @@ pub trait ReferenceAccessExt: easy::Access + Sized {
                 target: id.into(),
                 peeled: None,
             },
-            access: self,
+            handle: self,
         })
     }
 
     /// Returns the currently set namespace for references, or `None` if it is not set.
     ///
-    /// Namespaces allow to partition references.
-    fn namespace(&self) -> Result<Option<git_ref::Namespace>, easy::borrow::repo::Error> {
-        self.repo().map(|repo| repo.deref().refs.namespace.clone())
+    /// Namespaces allow to partition references, and is configured per `Easy`.
+    pub fn namespace(&self) -> Option<&git_ref::Namespace> {
+        self.refs.namespace.as_ref()
     }
 
-    /// Remove the currently set reference namespace and return it.
-    fn clear_namespace(&mut self) -> Result<Option<git_ref::Namespace>, easy::borrow::repo::Error> {
-        self.repo_mut().map(|mut repo| repo.deref_mut().refs.namespace.take())
+    /// Remove the currently set reference namespace and return it, affecting only this `Easy`.
+    pub fn clear_namespace(&mut self) -> Option<git_ref::Namespace> {
+        self.refs.namespace.take()
     }
 
     /// Set the reference namespace to the given value, like `"foo"` or `"foo/bar"`.
-    fn set_namespace<'a, Name, E>(
+    ///
+    /// Note that this value is shared across all `Easy…` instances as the value is stored in the shared `Repository`.
+    pub fn set_namespace<'a, Name, E>(
         &mut self,
         namespace: Name,
-    ) -> Result<Option<git_ref::Namespace>, easy::reference::namespace::set::Error>
+    ) -> Result<Option<git_ref::Namespace>, git_validate::refname::Error>
     where
         Name: TryInto<PartialNameRef<'a>, Error = E>,
         git_validate::refname::Error: From<E>,
     {
         let namespace = git_ref::namespace::expand(namespace)?;
-        Ok(self.repo_mut()?.deref_mut().refs.namespace.replace(namespace))
+        Ok(self.refs.namespace.replace(namespace))
     }
 
     // TODO: more tests or usage
@@ -88,13 +87,13 @@ pub trait ReferenceAccessExt: easy::Access + Sized {
     /// during creation and writing `log_message` into the reflog. Note that a ref-log will be written even if `log_message` is empty.
     ///
     /// The newly created Reference is returned.
-    fn reference<Name, E>(
+    pub fn reference<Name, E>(
         &self,
         name: Name,
         target: impl Into<ObjectId>,
         constraint: PreviousValue,
         log_message: impl Into<BString>,
-    ) -> Result<Reference<'_, Self>, reference::edit::Error>
+    ) -> Result<Reference<'_>, reference::edit::Error>
     where
         Name: TryInto<FullName, Error = E>,
         reference::edit::Error: From<E>,
@@ -136,7 +135,7 @@ pub trait ReferenceAccessExt: easy::Access + Sized {
     ///
     /// One or more `RefEdit`s  are returned - symbolic reference splits can cause more edits to be performed. All edits have the previous
     /// reference values set to the ones encountered at rest after acquiring the respective reference's lock.
-    fn edit_reference(
+    pub fn edit_reference(
         &self,
         edit: RefEdit,
         lock_mode: lock::acquire::Fail,
@@ -150,7 +149,7 @@ pub trait ReferenceAccessExt: easy::Access + Sized {
     ///
     /// Returns all reference edits, which might be more than where provided due the splitting of symbolic references, and
     /// whose previous (_old_) values are the ones seen on in storage after the reference was locked.
-    fn edit_references(
+    pub fn edit_references(
         &self,
         edits: impl IntoIterator<Item = RefEdit>,
         lock_mode: lock::acquire::Fail,
@@ -160,12 +159,11 @@ pub trait ReferenceAccessExt: easy::Access + Sized {
         let committer = match log_committer {
             Some(c) => c,
             None => {
-                committer_storage = self.committer()?;
+                committer_storage = self.committer();
                 &committer_storage
             }
         };
-        let repo = self.repo()?;
-        repo.refs
+        self.refs
             .transaction()
             .prepare(edits, lock_mode)?
             .commit(committer)
@@ -175,7 +173,7 @@ pub trait ReferenceAccessExt: easy::Access + Sized {
     /// Return the repository head, an abstraction to help dealing with the `HEAD` reference.
     ///
     /// The `HEAD` reference can be in various states, for more information, the documentation of [`Head`][easy::Head].
-    fn head(&self) -> Result<easy::Head<'_, Self>, reference::find::existing::Error> {
+    pub fn head(&self) -> Result<easy::Head<'_>, reference::find::existing::Error> {
         let head = self.find_reference("HEAD")?;
         Ok(match head.inner.target {
             Target::Symbolic(branch) => match self.find_reference(branch.to_partial()) {
@@ -194,9 +192,9 @@ pub trait ReferenceAccessExt: easy::Access + Sized {
     /// Find the reference with the given partial or full `name`, like `main`, `HEAD`, `heads/branch` or `origin/other`,
     /// or return an error if it wasn't found.
     ///
-    /// Consider [`try_find_reference(…)`][ReferenceAccessExt::try_find_reference()] if the reference might not exist
+    /// Consider [`try_find_reference(…)`][easy::Handle::try_find_reference()] if the reference might not exist
     /// without that being considered an error.
-    fn find_reference<'a, Name, E>(&self, name: Name) -> Result<Reference<'_, Self>, reference::find::existing::Error>
+    pub fn find_reference<'a, Name, E>(&self, name: Name) -> Result<Reference<'_>, reference::find::existing::Error>
     where
         Name: TryInto<PartialNameRef<'a>, Error = E>,
         git_ref::file::find::Error: From<E>,
@@ -209,32 +207,24 @@ pub trait ReferenceAccessExt: easy::Access + Sized {
     ///
     /// Common kinds of iteration are [all][easy::reference::iter::Platform::all()] or [prefixed][easy::reference::iter::Platform::prefixed()]
     /// references.
-    fn references(&self) -> Result<easy::reference::iter::Platform<'_, Self>, easy::reference::iter::Error> {
-        let state = self.state();
-        let repo = self.repo()?;
-        let packed_refs = state.assure_packed_refs_uptodate(&repo.refs)?;
+    pub fn references(&self) -> Result<easy::reference::iter::Platform<'_>, easy::reference::iter::Error> {
         Ok(easy::reference::iter::Platform {
-            repo,
-            packed_refs,
-            access: self,
+            platform: self.refs.iter()?,
+            handle: self,
         })
     }
 
     /// Try to find the reference named `name`, like `main`, `heads/branch`, `HEAD` or `origin/other`, and return it.
     ///
     /// Otherwise return `None` if the reference wasn't found.
-    /// If the reference is expected to exist, use [`find_reference()`][ReferenceAccessExt::find_reference()].
-    fn try_find_reference<'a, Name, E>(&self, name: Name) -> Result<Option<Reference<'_, Self>>, reference::find::Error>
+    /// If the reference is expected to exist, use [`find_reference()`][easy::Handle::find_reference()].
+    pub fn try_find_reference<'a, Name, E>(&self, name: Name) -> Result<Option<Reference<'_>>, reference::find::Error>
     where
         Name: TryInto<PartialNameRef<'a>, Error = E>,
         git_ref::file::find::Error: From<E>,
     {
-        let state = self.state();
-        let repo = self.repo()?;
-        match repo
-            .refs
-            .try_find(name, state.assure_packed_refs_uptodate(&repo.refs)?.buffer.as_ref())
-        {
+        let state = self;
+        match state.refs.try_find(name) {
             Ok(r) => match r {
                 Some(r) => Ok(Some(Reference::from_ref(r, self))),
                 None => Ok(None),
@@ -243,5 +233,3 @@ pub trait ReferenceAccessExt: easy::Access + Sized {
         }
     }
 }
-
-impl<A> ReferenceAccessExt for A where A: easy::Access + Sized {}
