@@ -1,5 +1,5 @@
 mod access {
-    use crate::{Kind, Repository};
+    use crate::{easy, Kind, Repository};
 
     impl Repository {
         /// Return the kind of repository, either bare or one with a work tree.
@@ -9,6 +9,11 @@ mod access {
                 None => Kind::Bare,
             }
         }
+
+        /// Add thread-local state to an easy-to-use handle for the most convenient API.
+        pub fn to_easy(&self) -> easy::Handle {
+            self.into()
+        }
     }
 }
 
@@ -22,7 +27,7 @@ mod from_path {
 
         fn try_from(value: Path) -> Result<Self, Self::Error> {
             let (git_dir, worktree_dir) = value.into_repository_and_work_tree_directories();
-            crate::Repository::open_from_paths(git_dir, worktree_dir)
+            crate::Repository::open_from_paths(git_dir, worktree_dir, Default::default())
         }
     }
 }
@@ -31,10 +36,34 @@ mod from_path {
 pub mod open {
     use std::{borrow::Cow, path::PathBuf};
 
-    use git_config::values::{Boolean, Integer};
+    use git_config::{
+        file::GitConfig,
+        values::{Boolean, Integer},
+    };
+    use git_features::threading::OwnShared;
 
     use crate::Repository;
 
+    /// The options used in [`Repository::open_opts
+    #[derive(Default)]
+    pub struct Options {
+        object_store_slots: git_odb::store::init::Slots,
+    }
+
+    impl Options {
+        /// Set the amount of slots to use for the object database. It's a value that doesn't need changes on the client, typically,
+        /// but should be controlled on the server.
+        pub fn object_store_slots(mut self, slots: git_odb::store::init::Slots) -> Self {
+            self.object_store_slots = slots;
+            self
+        }
+
+        /// Open a repository at `path` with the options set so far.
+        pub fn open(self, path: impl Into<std::path::PathBuf>) -> Result<Repository, Error> {
+            Repository::open_opts(path, self)
+        }
+    }
+
     /// The error returned by [`Repository::open()`].
     #[derive(Debug, thiserror::Error)]
     #[allow(missing_docs)]
@@ -44,7 +73,7 @@ pub mod open {
         #[error(transparent)]
         NotARepository(#[from] crate::path::is::Error),
         #[error(transparent)]
-        ObjectStoreInitialization(#[from] git_odb::linked::init::Error),
+        ObjectStoreInitialization(#[from] std::io::Error),
         #[error("Cannot handle objects formatted as {:?}", .name)]
         UnsupportedObjectFormat { name: crate::bstr::BString },
     }
@@ -52,6 +81,11 @@ pub mod open {
     impl Repository {
         /// Open a git repository at the given `path`, possibly expanding it to `path/.git` if `path` is a work tree dir.
         pub fn open(path: impl Into<std::path::PathBuf>) -> Result<Self, Error> {
+            Self::open_opts(path, Options::default())
+        }
+
+        /// Open a git repository at the given `path`, possibly expanding it to `path/.git` if `path` is a work tree dir.
+        fn open_opts(path: impl Into<std::path::PathBuf>, options: Options) -> Result<Self, Error> {
             let path = path.into();
             let (path, kind) = match crate::path::is::git(&path) {
                 Ok(kind) => (path, kind),
@@ -62,27 +96,26 @@ pub mod open {
             };
             let (git_dir, worktree_dir) =
                 crate::Path::from_dot_git_dir(path, kind).into_repository_and_work_tree_directories();
-            Repository::open_from_paths(git_dir, worktree_dir)
+            Repository::open_from_paths(git_dir, worktree_dir, options)
         }
 
         pub(in crate::repository) fn open_from_paths(
             git_dir: PathBuf,
             mut worktree_dir: Option<PathBuf>,
+            Options { object_store_slots }: Options,
         ) -> Result<Self, Error> {
             let config = git_config::file::GitConfig::open(git_dir.join("config"))?;
             if worktree_dir.is_none() {
-                let is_bare = config
-                    .value::<Boolean<'_>>("core", None, "bare")
-                    .map_or(false, |b| matches!(b, Boolean::True(_)));
+                let is_bare = config_bool(&config, "core.bare", false);
                 if !is_bare {
                     worktree_dir = Some(git_dir.parent().expect("parent is always available").to_owned());
                 }
             }
-            let hash_kind = if config
+            let use_multi_pack_index = config_bool(&config, "core.multiPackIndex", true);
+            let repo_format_version = config
                 .value::<Integer>("core", None, "repositoryFormatVersion")
-                .map_or(0, |v| v.value)
-                == 1
-            {
+                .map_or(0, |v| v.value);
+            let object_hash = if repo_format_version == 1 {
                 if let Ok(format) = config.value::<Cow<'_, [u8]>>("extensions", None, "objectFormat") {
                     match format.as_ref() {
                         b"sha1" => git_hash::Kind::Sha1,
@@ -100,20 +133,35 @@ pub mod open {
             };
 
             Ok(crate::Repository {
-                odb: git_odb::linked::Store::at(git_dir.join("objects"))?,
-                refs: git_ref::file::Store::at(
+                objects: OwnShared::new(git_odb::Store::at_opts(
+                    git_dir.join("objects"),
+                    git_odb::store::init::Options {
+                        slots: object_store_slots,
+                        object_hash,
+                        use_multi_pack_index,
+                    },
+                )?),
+                refs: crate::RefStore::at(
                     git_dir,
                     if worktree_dir.is_none() {
-                        git_ref::file::WriteReflog::Disable
+                        git_ref::store::WriteReflog::Disable
                     } else {
-                        git_ref::file::WriteReflog::Normal
+                        git_ref::store::WriteReflog::Normal
                     },
+                    object_hash,
                 ),
                 work_tree: worktree_dir,
-                hash_kind,
+                object_hash,
             })
         }
     }
+
+    fn config_bool(config: &GitConfig<'_>, key: &str, default: bool) -> bool {
+        let (section, key) = key.split_once(".").expect("valid section.key format");
+        config
+            .value::<Boolean<'_>>(section, None, key)
+            .map_or(default, |b| matches!(b, Boolean::True(_)))
+    }
 }
 
 ///
@@ -150,14 +198,14 @@ mod location {
     impl Repository {
         /// The path to the `.git` directory itself, or equivalent if this is a bare repository.
         pub fn path(&self) -> &std::path::Path {
-            &self.refs.base
+            self.git_dir()
         }
 
         /// Return the path to the repository itself, containing objects, references, configuration, and more.
         ///
         /// Synonymous to [`path()`][Repository::path()].
         pub fn git_dir(&self) -> &std::path::Path {
-            &self.refs.base
+            self.refs.base()
         }
 
         /// Return the path to the working directory if this is not a bare repository.
@@ -167,7 +215,7 @@ mod location {
 
         /// Return the path to the directory containing all objects.
         pub fn objects_dir(&self) -> &std::path::Path {
-            &self.odb.dbs[0].loose.path
+            self.objects.path()
         }
     }
 }
